@@ -44,20 +44,35 @@ extern "C" {
  * @{
  */
 
-#ifdef CONF_SYMBOL_VISIBILITY
-#   if defined PLATFORM_WINDOWS
-#       define LF_EXPORT    __declspec(dllexport)
-#   elif defined CONF_COMPILER_GCC
-#       define LF_EXPORT    __attribute__((visibility("default")))
-#   else
-#       error "I don't know how to change symbol visibility for your compiler"
-#   endif
+/// Major library version number
+#define LF_VERSION_MAJOR	0
+/// Minor library version number
+#define LF_VERSION_MINOR	2
+/// Library micro version number
+#define LF_VERSION_MICRO	7
+/// Library bugfix number
+#define LF_VERSION_BUGFIX	0
+/// Full library version
+#define LF_VERSION	((LF_VERSION_MAJOR << 24) | (LF_VERSION_MINOR << 16) | (LF_VERSION_MICRO << 8) | LF_VERSION_BUGFIX)
+
+#if defined CONF_LENSFUN_STATIC
+/// This macro expands to an appropiate symbol visibility declaration
+#   define LF_EXPORT
 #else
-#   if defined CONF_COMPILER_MSVC
-#       define LF_EXPORT    __declspec(dllimport)
+#   ifdef CONF_SYMBOL_VISIBILITY
+#       if defined PLATFORM_WINDOWS
+#           define LF_EXPORT    __declspec(dllexport)
+#       elif defined CONF_COMPILER_GCC
+#           define LF_EXPORT    __attribute__((visibility("default")))
+#       else
+#           error "I don't know how to change symbol visibility for your compiler"
+#       endif
 #   else
-        /// This macro expands to an appropiate symbol visibility declaration
-#       define LF_EXPORT
+#       if defined PLATFORM_WINDOWS || defined _MSC_VER
+#           define LF_EXPORT    __declspec(dllimport)
+#       else
+#           define LF_EXPORT
+#       endif
 #   endif
 #endif
 
@@ -385,7 +400,7 @@ LF_EXPORT cbool lf_camera_check (lfCamera *camera);
 /**
  * The lensdb library implements several lens distortion models.
  * This enum lists them. Distortion usually heavily depends on the
- * focal distance, but does not depend on the aperture.
+ * focal length, but does not depend on the aperture.
  *
  * For a popular explanation of lens distortion see
  * http://www.vanwalree.com/optics/distortion.html
@@ -396,7 +411,7 @@ enum lfDistortionModel
     LF_DIST_MODEL_NONE,
     /**
      * 3rd order polynomial model:
-     * Ru = Rd * (1 + k1 * Rd^2)
+     * Ru = Rd * (1 - k1 + k1 * Rd^2)
      * Ref: http://www.imatest.com/docs/distortion.html
      */
     LF_DIST_MODEL_POLY3,
@@ -413,7 +428,7 @@ enum lfDistortionModel
     LF_DIST_MODEL_FOV1,
     /**
      * PTLens rectilinear:
-     * Ru = Rd * (a * Rd^3 + b * Rd^2 + c * Rd + 1)
+     * Ru = Rd * (a * Rd^3 + b * Rd^2 + c * Rd + 1 - a - b - c)
      */
     LF_DIST_MODEL_PTLENS,
 };
@@ -422,14 +437,14 @@ C_TYPEDEF (enum, lfDistortionModel)
 
 /**
  * Lens distortion calibration data. Lens distortion depends only
- * of focal distance. The library will interpolate the coefficients
- * values if data for the exact focal distance is not available.
+ * of focal length. The library will interpolate the coefficients
+ * values if data for the exact focal length is not available.
  */
 struct lfLensCalibDistortion
 {
     /** The type of distortion model used */
     enum lfDistortionModel Model;
-    /** Focal distance at which this calibration data was taken (0 - unspecified) */
+    /** Focal length at which this calibration data was taken (0 - unspecified) */
     float Focal;
     /** Distortion coefficients, dependent on model (a,b,c; k1,k2 or omega) */
     float Terms [3];
@@ -440,7 +455,7 @@ C_TYPEDEF (struct, lfLensCalibDistortion)
 /**
  * The lensdb library supports several models for lens lateral
  * chromatic aberrations (also called transversal chromatic
- * aberrations, TCA). TCAs depend on focal distance, but does not
+ * aberrations, TCA). TCAs depend on focal length, but does not
  * depend of the aperture.
  *
  * For a popular explanation of chromatic aberrations see
@@ -471,15 +486,15 @@ C_TYPEDEF (enum, lfTCAModel)
 
 /**
  * Laterlal chromatic aberrations calibration data. Chromatic aberrations
- * depend on focal distance and aperture value. The library will interpolate
- * the coefficients if data for the exact focal distance and aperture value
- * is not available with priority for a more exact focal distance.
+ * depend on focal length and aperture value. The library will interpolate
+ * the coefficients if data for the exact focal length and aperture value
+ * is not available with priority for a more exact focal length.
  */
 struct lfLensCalibTCA
 {
     /** The lateral chromatic aberration model used */
     enum lfTCAModel Model;
-    /** Focal distance at which this calibration data was taken (0 - unspecified) */
+    /** Focal length at which this calibration data was taken (0 - unspecified) */
     float Focal;
     /** The coefficients for TCA, dependent on model; separate for R and B */
     float Terms [6];
@@ -493,7 +508,7 @@ C_TYPEDEF (struct, lfLensCalibTCA)
  * can be generalized for all lenses of a certain type; mechanical
  * vignetting is out of the scope of this library.
  *
- * Vignetting is dependent on both focal distance and aperture.
+ * Vignetting is dependent on both focal length and aperture.
  *
  * For a popular explanation of vignetting see
  * http://www.vanwalree.com/optics/vignetting.html
@@ -514,17 +529,17 @@ enum lfVignettingModel
 C_TYPEDEF (enum, lfVignettingModel)
 
 /**
- * Lens vignetting calibration data. Vignetting depends on focal distance,
+ * Lens vignetting calibration data. Vignetting depends on focal length,
  * aperture and distance to subject. The library will interpolate
- * the coefficients if data for the exact focal distance, aperture
+ * the coefficients if data for the exact focal length, aperture
  * and distance-to-subject is not available, trying first to match
- * the exact focal distance, then the aperture.
+ * the exact focal length, then the aperture.
  */
 struct lfLensCalibVignetting
 {
     /** The lens vignetting model used */
     enum lfVignettingModel Model;
-    /** Focal distance at which this calibration data was taken (0 - unspecified) */
+    /** Focal length at which this calibration data was taken (0 - unspecified) */
     float Focal;
     /** Aperture at which this calibration data was taken (0 - unspecified) */
     float Aperture;
@@ -536,6 +551,61 @@ struct lfLensCalibVignetting
 
 C_TYPEDEF (struct, lfLensCalibVignetting)
 
+/**
+ *  Different crop modes
+ */
+enum lfCropMode
+{
+    /** no crop at all */
+    LF_NO_CROP,
+    /** use a rectangular crop */
+    LF_CROP_RECTANGLE,
+    /** use a circular crop, e.g. for circular fisheye images */
+    LF_CROP_CIRCLE
+};
+
+C_TYPEDEF(enum, lfCropMode)
+
+/**
+ *  Struct to save image crop, which can depend on the focal length
+ */
+struct lfLensCalibCrop
+{
+    /** Focal length at which this calibration data was taken (0 - unspecified) */
+    float Focal;
+    /** crop mode which should be applied to image to get rid of black borders */
+    enum lfCropMode CropMode;
+    /** @brief Crop coordinates, relative to image corresponding image dimension 
+     *
+     *  Crop goes left - 0, right - 1, top - 2, bottom - 3 
+     *  Left/right refers always to long side (width in landscape mode), 
+     *  top bottom to short side (height in landscape).
+     *  Also negative values are allowed for cropping of fisheye images,
+     *  where the crop circle can extend above the image border.
+     */
+    float Crop [4];
+};
+
+C_TYPEDEF (struct, lfLensCalibCrop)
+
+/**
+ *  Struct to save calibrated field of view, which can depends on the focal length
+ */
+struct lfLensCalibFov
+{
+    /** Focal length at which this calibration data was taken (0 - unspecified) */
+    float Focal;
+    /** @brief Field of view for given images
+     * 
+     *  Especially for fisheye images the field of view calculated from the (EXIF) focal
+     *  length differs slightly from the real field of view. The real field of view can be 
+     *  stored in this field 
+     */
+    float FieldOfView;
+};
+
+C_TYPEDEF (struct, lfLensCalibFov)
+
 /**
  * This structure describes a single parameter for some lens model.
  */
@@ -573,7 +643,15 @@ enum lfLensType
      * Equirectangular (not that there are such lenses, but useful
      * to convert images TO this type, especially fish-eye images.
      */
-    LF_EQUIRECTANGULAR
+    LF_EQUIRECTANGULAR,
+    /** orthographic fisheye */
+    LF_FISHEYE_ORTHOGRAPHIC,
+    /** stereographic fisheye */
+    LF_FISHEYE_STEREOGRAPHIC,
+    /** equisolid fisheye */
+    LF_FISHEYE_EQUISOLID,
+    /** fisheye Thoby (for Nikkor 10.5) */
+    LF_FISHEYE_THOBY
 };
 
 C_TYPEDEF (enum, lfLensType)
@@ -594,13 +672,13 @@ struct LF_EXPORT lfLens
     lfMLstr Maker;
     /** Lens model (ex: "Zoom-Rolleinar") */
     lfMLstr Model;
-    /** Minimum focal distance, mm (ex: 35). */
+    /** Minimum focal length, mm (ex: 35). */
     float MinFocal;
-    /** Maximum focal distance, mm (ex: 105). Can be equal to MinFocal. */
+    /** Maximum focal length, mm (ex: 105). Can be equal to MinFocal. */
     float MaxFocal;
-    /** Aperture at minimum focal distance (ex: 3.5). */
+    /** Aperture at minimum focal length (ex: 3.5). */
     float MinAperture;
-    /** Aperture at maximum focal distance (ex: 4.3). Can be equal to MinAperture. */
+    /** Aperture at maximum focal length (ex: 4.3). Can be equal to MinAperture. */
     float MaxAperture;
     /** Available mounts (NULL-terminated list) (ex: { "QBM", NULL }) */
     char **Mounts;
@@ -632,6 +710,10 @@ struct LF_EXPORT lfLens
     lfLensCalibTCA **CalibTCA;
     /** Lens vignetting calibration data, NULL-terminated (unsorted) */
     lfLensCalibVignetting **CalibVignetting;
+    /** Crop data, NULL-terminated (unsorted) */
+    lfLensCalibCrop **CalibCrop;
+    /** Field of view calibration data, NULL-terminated (unsorted) */
+    lfLensCalibFov **CalibFov;
     /** Lens matching score, used while searching: not actually a lens parameter */
     int Score;
 
@@ -732,10 +814,42 @@ struct LF_EXPORT lfLens
      */
     bool RemoveCalibVignetting (int idx);
 
+    /**
+     * Add a new lens crop structure to the pool.
+     * The objects is copied, thus you can reuse it as soon as
+     * this function returns.
+     * @param cc 
+     *     The lens crop structure.
+     */
+    void AddCalibCrop (const lfLensCalibCrop *cc);
+
+    /**
+     * Remove a lens crop entry from the lens crop structure.
+     * @param idx
+     *     The lens crop data index (zero-based).
+     */
+    bool RemoveCalibCrop (int idx);
+
+    /**
+     * Add a new lens fov structure to the pool.
+     * The objects is copied, thus you can reuse it as soon as
+     * this function returns.
+     * @param cf
+     *     The lens fov structure.
+     */
+    void AddCalibFov (const lfLensCalibFov *cf);
+
+    /**
+     * Remove a field of view  entry from the lens fov structure.
+     * @param idx
+     *     The lens information data index (zero-based).
+     */
+    bool RemoveCalibFov (int idx);
+
     /**
      * This method fills some fields if they are missing but
      * can be derived from other fields. This includes such non-obvious
-     * parameters like the range of focal distances or the range
+     * parameters like the range of focal lengths or the range
      * of apertures, which can be derived from lens named (which is
      * intelligently parsed) or from the list of calibrations.
      */
@@ -801,6 +915,24 @@ struct LF_EXPORT lfLens
     static const char *GetVignettingModelDesc (
         lfVignettingModel model, const char **details, const lfParameter ***params);
 
+    /**
+     * Get the human-readable crop name and the descriptions
+     * of the parameters required by this model.
+     * @param mode
+     *     The crop mode.
+     * @param details
+     *     If not NULL, this string will be set to a more detailed (technical)
+     *     description of the model. This string may contain newlines.
+     * @param params
+     *     If not NULL, this variable will be set to a pointer to an array
+     *     of lfParameter structures, every structure describes a model
+     *     parameter. The list is NULL-terminated.
+     * @return
+     *     A short name of the distortion model or NULL if model is unknown.
+     */
+    static const char *GetCropDesc (
+        lfCropMode mode, const char **details, const lfParameter ***params);
+
     /**
      * Get the human-readable lens type name and a short description of this
      * lens type.
@@ -817,7 +949,7 @@ struct LF_EXPORT lfLens
     /**
      * Interpolate lens geometry distortion data for given focal length.
      * @param focal
-     *     The focal distance at which we need geometry distortion parameters.
+     *     The focal length at which we need geometry distortion parameters.
      * @param res
      *     The resulting interpolated model.
      */
@@ -826,26 +958,44 @@ struct LF_EXPORT lfLens
     /**
      * Interpolate lens TCA calibration data for given focal length.
      * @param focal
-     *     The focal distance at which we need TCA parameters.
+     *     The focal length at which we need TCA parameters.
      * @param res
      *     The resulting interpolated model.
      */
     bool InterpolateTCA (float focal, lfLensCalibTCA &res) const;
 
     /**
-     * Interpolate lens vignetting model parameters for given focal distance,
-     * aperture and focus distance.
+     * Interpolate lens vignetting model parameters for given focal length,
+     * aperture and subject distance.
      * @param focal
-     *     The focal distance for which we need vignetting parameters.
+     *     The focal length for which we need vignetting parameters.
      * @param aperture
      *     The aperture value for which we need vignetting parameters.
      * @param distance
-     *     The focus distance for which we need vignetting parameters.
+     *     The subject distance for which we need vignetting parameters.
      * @param res
      *     The resulting interpolated model.
      */
     bool InterpolateVignetting (
         float focal, float aperture, float distance, lfLensCalibVignetting &res) const;
+
+    /**
+     * Interpolate lens crop data for given focal length.
+     * @param focal
+     *     The focal length at which we need image parameters.
+     * @param res
+     *     The resulting interpolated information data.
+     */
+    bool InterpolateCrop (float focal, lfLensCalibCrop &res) const;
+
+    /**
+     * Interpolate lens fov data for given focal length.
+     * @param focal
+     *     The focal length at which we need image parameters.
+     * @param res
+     *     The resulting interpolated information data.
+     */
+    bool InterpolateFov (float focal, lfLensCalibFov &res) const;
 #endif
 };
 
@@ -899,6 +1049,10 @@ LF_EXPORT const char *lf_get_tca_model_desc (
 LF_EXPORT const char *lf_get_vignetting_model_desc (
     enum lfVignettingModel model, const char **details, const lfParameter ***params);
 
+/** @sa lfLens::GetCropDesc */
+LF_EXPORT const char *lf_get_crop_desc (
+    enum lfCropMode mode, const char **details, const lfParameter ***params);
+
 /** @sa lfLens::GetLensTypeDesc */
 LF_EXPORT const char *lf_get_lens_type_desc (
     enum lfLensType type, const char **details);
@@ -914,6 +1068,14 @@ LF_EXPORT cbool lf_lens_interpolate_tca (const lfLens *lens, float focal, lfLens
 LF_EXPORT cbool lf_lens_interpolate_vignetting (const lfLens *lens, float focal, float aperture,
     float distance, lfLensCalibVignetting *res);
 
+/** @sa lfLens::InterpolateCrop */
+LF_EXPORT cbool lf_lens_interpolate_crop (const lfLens *lens, float focal,
+    lfLensCalibCrop *res);
+
+/** @sa lfLens::InterpolateFov */
+LF_EXPORT cbool lf_lens_interpolate_fov (const lfLens *lens, float focal,
+    lfLensCalibFov *res);
+
 /** @sa lfLens::AddCalibDistortion */
 LF_EXPORT void lf_lens_add_calib_distortion (lfLens *lens, const lfLensCalibDistortion *dc);
 
@@ -932,6 +1094,18 @@ LF_EXPORT void lf_lens_add_calib_vignetting (lfLens *lens, const lfLensCalibVign
 /** @sa lfLens::RemoveCalibVignetting */
 LF_EXPORT cbool lf_lens_remove_calib_vignetting (lfLens *lens, int idx);
 
+/** @sa lfLens::AddCalibCrop */
+LF_EXPORT void lf_lens_add_calib_crop (lfLens *lens, const lfLensCalibCrop *cc);
+
+/** @sa lfLens::RemoveCalibCrop */
+LF_EXPORT cbool lf_lens_remove_calib_crop (lfLens *lens, int idx);
+
+/** @sa lfLens::AddCalibFov */
+LF_EXPORT void lf_lens_add_calib_fov (lfLens *lens, const lfLensCalibFov *cf);
+
+/** @sa lfLens::RemoveCalibFov */
+LF_EXPORT cbool lf_lens_remove_calib_fov (lfLens *lens, int idx);
+
 /** @} */
 
 /*----------------------------------------------------------------------------*/
@@ -1713,8 +1887,8 @@ struct LF_EXPORT lfModifier
 
     /**
      * Add the stock lens distortion correcting callback into the chain.
-     * The distortion correction callback always has a fixed priority of 250
-     * when rectifying a image and 750 on reverse transform.
+     * The distortion correction callback always has a fixed priority of 750
+     * when rectifying a image and 250 on reverse transform.
      * @param model
      *     Lens distortion model data.
      * @param reverse
@@ -1733,7 +1907,7 @@ struct LF_EXPORT lfModifier
      * @param to
      *     The lens model for target image.
      * @param focal
-     *     Lens focal distance.
+     *     Lens focal length.
      * @return
      *     True if a library has a callback for given from->to conversion.
      */
